#
# Copyright INRA-URGI 2009-2010
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
# 
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
#
import re
import sys
from commons.core.parsing.MapperParser import MapperParser
from SMART.Java.Python.structure.Interval import Interval
from SMART.Java.Python.structure.SubMapping import SubMapping
from SMART.Java.Python.structure.Mapping import Mapping


class BlastParser(MapperParser):
    """A class that parses the output of Blast (-m 8 format)"""

    def __init__(self, fileName, verbosity = 0):
        super(BlastParser, self).__init__(fileName, verbosity)


    def __del__(self):
        super(BlastParser, self).__del__()


    def getFileFormats():
        return ["blast"]
    getFileFormats = staticmethod(getFileFormats)


    def skipFirstLines(self):
        pass


    def parseLine(self, line):
        m = re.search(r"^(\S+)\s+(\S+)\s+(\d+\.?\d*)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+([-+]?\d+\.?\d*[eE]?[-+]?\d*)\s+(\d+\.?\d*)\s*$", line)
        if m == None:
            sys.exit("\nLine %d '%s' does not have an Blast format" % (self.currentLineNb, line))

        mapping = Mapping()

        queryInterval = Interval()
        queryInterval.setName(m.group(1))
        queryInterval.setStart(min(int(m.group(7)), int(m.group(8))))
        queryInterval.setEnd(max(int(m.group(7)), int(m.group(8))))

        targetInterval = Interval()
        targetInterval.setChromosome(m.group(2))
        targetInterval.setStart(min(int(m.group(9)), int(m.group(10))))
        targetInterval.setEnd(max(int(m.group(9)), int(m.group(10))))

        subMapping = SubMapping()
        subMapping.setQueryInterval(queryInterval)
        subMapping.setTargetInterval(targetInterval)

        mapping.addSubMapping(subMapping)

        mapping.setIdentity(round(float(m.group(3))))
        mapping.setSize(int(m.group(4)))
        mapping.setNbMismatches(int(m.group(5)))
        mapping.setNbGaps(int(m.group(6)))
        mapping.setDirection((int(m.group(8)) - int(m.group(7))) * (int(m.group(10)) - int(m.group(9))))
        mapping.setEvalue(float(m.group(11)))

        return mapping
